/*
 * Copyright (C) FEIG ELECTRONIC GmbH. All rights reserved. Do not disclose.
 *
 * This software is the confidential and proprietary information of
 * FEIG ELECTRONIC GmbH ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with FEIG ELECTRONIC GmbH.
 */

#ifndef FEDM_FUNIT_MODULE_H
#define FEDM_FUNIT_MODULE_H

#include <Fedm.h>
#include <FedmFunitBasic.h>
#include <FedmFunitElements.h>
#include <FedmFunitHide.h>
#include <GroupFunit.h>
#include <GroupPdevice.h>



namespace FEDM {
namespace FunctionUnit {

/// @brief MAIN class for function unit and peripheral device extensions of a ReaderModule
class FEDM_FUNIT_CLASS_DEF ExtensionModule
{
public:
/** @name Main
 */
///@{
	/// @brief Constructor for the function unit module
	/// @param[in] reader A pointer to a ReaderModule
	///
	/// The function unit module does not take ownership of the reader module.<br>
	/// The function unit module must be destroyed before reader module's destructor is called !<br>
	ExtensionModule(ReaderModule *reader);

	/// @brief The standard constructor is not allowed
	ExtensionModule() = delete;

	/// @brief To copy a ExtensionModule is not allowed
	ExtensionModule(ExtensionModule & other) = delete;

	/// @brief To copy a ExtensionModule is not allowed
	ExtensionModule(const ExtensionModule & other) = delete;

	/// @brief To copy a ExtensionModule is not allowed
	ExtensionModule & operator=(ExtensionModule & other) = delete;

	/// @brief To copy a ExtensionModule is not allowed
	ExtensionModule & operator=(const ExtensionModule & other) = delete;

	~ExtensionModule();

	/// @brief Query the version the fedm-service library
	static std::string libVersion(void);
///@}



// ############################################################# Error Handling #############################################################
/** @name Error Handling
 */
///@{
	/// @brief Get last library error code
	/// @return The last library error code
	///
	/// If the last library function call delivered a negative return code this function will return this value.<br>
	/// Otherwise 0 will be returned.
	int lastError(void) const;

	/// @brief Get status code of last reader response
	/// @return The last reader status #{DOC_LINK_TO_CLASS(FEDM::ReaderStatus)}
	///
	/// If the last library function call delivered a positive return code this function will return the LSB of that.
	int lastReaderStatus(void) const;

	/// @brief Get status code of last function unit response
	/// @return The last function unit status #{DOC_LINK_TO_CLASS(FEDM::ReaderStatus)}
	///
	/// If the last library function call delivered a positive return code this function will return the 2nd LSB of that.
	int lastFunctionUnitStatus(void) const;

	/// @brief Get last error or status text
	/// @return The text of last library error, reader status or function unit status
	std::string lastErrorStatusText(void) const;
///@}


// ####################################################### Extension Function Groups #######################################################
/** @name Extension Function Groups Non-Const
 */
///@{
	Funit   & funit  (void) { return m_funit; }
	Pdevice & pdevice(void) { return m_pdevice; }
///@}

/** @name Extension Function Groups Const
 */
///@{
	const Funit   & funit  (void) const { return m_funit; }
	const Pdevice & pdevice(void) const { return m_pdevice; }
///@}


private:
	Internal::FunitWorker *m_funitWorker;

	Funit   m_funit;
	Pdevice m_pdevice;

}; // end of class ExtensionModule

}; // end of namespace FunctionUnit
}; // end of namespace FEDM

#endif // FEDM_FUNIT_MODULE_H




/*

Adressvergabe bei dynamischen (DAT-Tuner) Antennen:
Das ist nur notwendig, wenn diese hinter Power-Splittern sind.
Ansonsten die Default-Adresse 1 verwenden! Ggf. muss diese wieder zurückgesetzt werden.

Vorgehensweise:
Alle Top-Level-Power-Splitter finden. Das sind die Elemente FunitHfPwrSplitter oder FunitHfMuxDual4.

Folgendes muss für jedes dieser Elemente durchgeführt werden:

- Alle FunitHfAntDynamic-Elemente ab dem jeweiligen Top-Level-Power-Splitter zählen. Es müssen 2 oder 4 sein.
  Ansonsten Fehler
  
- Alle FunitHfAntDynamic-Elemente in Auto-Adress-Mode setzen (Command 0xCB Set Mode 0x01 für die Tuner-Addr 1..4 schicken)
  Diese haben jetzt temporäre Zufallsadressen (5..127)
  
- Dann Command 0xCA Set Address OLD-ADDR NEW-ADDR schicken:
  OLD-ADDR von 5..127
  NEW-ADDR beginnend mit 1
  Wenn kein Fehler, NEW-ADDR++
  Vorgang wiederholen, bis alle 2 oder 4 FunitHfAntDynamic umadressiert sind.
  
- Command 0xC8 Store Settings für die Tuner-Addr 1..2/4 schicken.

- Command 0xC1 CPU-Reset für die Tuner-Addr 1..2/4 schicken.

Fertig.

Frage:
Damit dieser komplexe Kommunikationsvorgang funktionieren kann, müssen die Antennen abgeglichen sein.
Um die Antennen abzugleichen müssen aber die Adressen gestellt werden.
Wie soll dieses Kunststück gelingen ???

Dummerweise haben die DAT-Tuner keine Hardware-Jumper für die Adressvergabe (im Gegensatz zu den Multiplexern)
- sehr ungeschickt !!!

Nach der Addressvergabe gibt es keine eindeutige Zuordnung der Adressen 1..2/4 zu den DAT-Tunern.

*/
