/*
 * Copyright (C) FEIG ELECTRONIC GmbH. All rights reserved. Do not disclose.
 *
 * This software is the confidential and proprietary information of
 * FEIG ELECTRONIC GmbH ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with FEIG ELECTRONIC GmbH.
 */

#ifndef FEDM_GROUP_FUNIT_H
#define FEDM_GROUP_FUNIT_H

#include <FedmFunitBasic.h>
#include <FedmFunitElements.h>
#include <FedmFunitHide.h>



namespace FEDM {
namespace FunctionUnit {

/// @brief GROUP class for function unit support
///
/// When using function units, the reader type of the underlying ReaderModule must be set, i.e. a connection should be established, before the ExtensionModule is created!<br>
/// If the function readerRfChannelCount() returns 0 the reader type does not support function units.<br>
/// The reader should configured to HostMode. In other operation modes the reader will automatically scan the configured antennas.
class FEDM_FUNIT_CLASS_DEF Funit
{
    friend class ExtensionModule;

public:
	/// @brief Create a HF multiplexer - single 1 to 8
	/// @param[in] parentChannel The output channel of the parent which is connected to the multiplexers input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created multiplexer object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader or another HF multiplexer (single).<br>
	/// Valid output channels of this multiplexer are 1 ... 8.
	FunitHfMuxSingle *createHfMuxSingle_1to8(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Create a HF multiplexer - single 1 to 5
	/// @param[in] parentChannel The output channel of the parent which is connected to the multiplexers input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created multiplexer object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader or another HF multiplexer (single).<br>
	/// Valid output channels of this multiplexer are 1 ... 5.
	FunitHfMuxSingle *createHfMuxSingle_1to5(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Create a HF multiplexer - dual 1 to 4 - with a preconnected HF power splitter
	/// @param[in] parentChannel The output channel of the parent which is connected to the multiplexers input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created multiplexer object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader or a HF multiplexer (single).<br>
	/// Valid output channels of this multiplexer are 1 ... 8. These are paired.<br>
	/// The outputs of a pair will be switched together. The pairing is fixed: 1-8, 2-7, 3-6, 4-5<br>
	/// This HF multiplexer is preconnected with a HF power splitter.
	FunitHfMuxDual *createHfMuxDual_1to4(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Create a HF power splitter
	/// @param[in] parentChannel The output channel of the parent which is connected to the power splitters input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created power splitter object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader, a HF multiplexer (single or dual) or another HF power splitter.<br>
	/// Valid output channels are 1 and 2.
	FunitHfPwrSplitter *createHfPwrSplitter(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Create a HF antenna - dynamic (DAT tuner)
	/// @param[in] parentChannel The output channel of the parent which is connected to the antenna input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created antenna object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader, a HF multiplexer (single or dual) or a HF power splitter.
	FunitHfAntDynamic *createHfAntDynamic(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Create a HF antenna - static
	/// @param[in] parentChannel The output channel of the parent which is connected to the antenna input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created antenna object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader, a HF multiplexer (single or dual) or a HF power splitter.
	FunitHfAntStatic *createHfAntStatic(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Create a UHF multiplexer - single 1 to 8
	/// @param[in] parentChannel The output channel of the parent which is connected to the multiplexers input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created multiplexer object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader or another UHF multiplexer (single).<br>
	/// Valid output channels of this multiplexer are 1 ... 8.
	FunitUhfMuxSingle *createUhfMuxSingle_1to8(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Create a UHF antenna - static
	/// @param[in] parentChannel The output channel of the parent which is connected to the antenna input.
	/// @param[in] parentElement A pointer to the parent function unit or nullptr if parent is the reader.
	/// @return A pointer to the created antenna object. The ExtensionModule keeps ownership of this object.<br>
	/// A nullptr is returned if creation failed. Call lastErrorStatusText() for details.
	///
	/// Valid parents are the reader or an UHF multiplexer (single).
	FunitUhfAntStatic *createUhfAntStatic(size_t parentChannel, FunitElement *parentElement = nullptr);

	/// @brief Destroy all created function unit objects
	void removeAllUnits(void);

	/// @brief Query a first level function unit
	/// @return A pointer to the function unit element, which is connected to the given reader RF output channel<br>
	/// or nullptr if channel is out of range or no FunitElement is connected to the given channel.
	FunitElement *firstParent(size_t channel);

	/// @brief Query all antenna function units
	/// @return A vector of all antenna function units
	std::vector<FunitElement *> allAntennas(void);

	/// @brief Query all first level function units
	/// @return A vector of all function unit elements, which are conneted to a reader RF output.<br>
	/// In case of an unconnected output the vector contains a nullptr.<br>
	/// The index of the vector is reader RF channel - 1.
	std::vector<FunitElement *> allFirstParents(void);
	
	/// @brief Query the readers RF channel count
	/// @return The count of RF outputs or 0 if the reader type does not support function units or the underlying ReaderModule is in unconnected state (i.e. the reader type is not set).
	size_t readerRfChannelCount(void) const;

	/// @brief Activate / deactivate a readers RF output for DC power supply
	/// @param[in] channel The RF output channel number (1 ... readerRfChannelCount). Use 0 to set all channels.
	/// @param[in] on If true the appropriate RF output will supply DC power when switched on.
	int setReaderRfChannelDcPowerSupply(size_t channel, bool on);

	/// @brief Query a summary description of the total function unit equipment
	///
	/// Call this function after all function units are created.<br>
	/// All antennas will be selected sequentially and the firmware version information of all active function units will be read.
	std::string summary(void);

protected:
    Funit();

    Internal::FunitWorker *m_funitWorker; // set by a friend

}; // end of class Funit

}; // end of namespace FunctionUnit
}; // end of namespace FEDM

#endif // FEDM_GROUP_FUNIT_H
